import React from 'react'

import {withStyles} from '@material-ui/core/styles'
import classNames from "classnames"
import Toolbar from "../_base/Toolbar"
import IconButton from "../_base/IconButton"
import InputBase from "../_base/InputBase"
import ToolTip from "../_base/Tooltip"
import SearchIcon from '@material-ui/icons/Search'
import FilterListIcon from '@material-ui/icons/FilterList'


const styles = theme => ({
  root: {
    paddingRight: theme.spacing.unit,
  },
  highlight:
    theme.palette.type === 'light'
      ? {
        color: theme.palette.secondary.main,
        backgroundColor: lighten(theme.palette.secondary.light, 0.85),
      }
      : {
        color: theme.palette.text.primary,
        backgroundColor: theme.palette.secondary.dark,
      },
  spacer: {
    flex: '1 1 100%',
  },
  actions: {
    flex: '0 0 auto',
    color: theme.palette.text.secondary,
  },
  title: {
    flex: '0 0 auto',
  },

})

function TTOOLBAR(props) {
  const {
    title, inMoving, numSelected, classes,
    handleAdd, handleMove, handleDelete, handleSearch
  } = props

  return (
    <Toolbar
      className={classNames(classes.root, {
        [classes.highlight]: numSelected > 0,
      })}
    >
      <div className={classes.title}>
        <IconButton className={classes.iconButton} aria-label="Search">
          <SearchIcon/>
        </IconButton>
        <InputBase
          className={classes.input}
          placeholder="查找……"
          id="searchkey"
          onChange={handleSearch}
        />
      </div>
      <div className={classes.spacer}/>
      <div className={classes.actions}>
        {
          numSelected > 0 ? (
            <div>
              <ToolTip title="移动">
                <IconButton aria-label="Move" onClick={handleMove}>
                  <OpenWithIcon/>
                </IconButton>
              </ToolTip>
              {inMoving ? <span style={{animation: 'cursorjump 1s infinite forwards'}}>请选择目标分类……</span> : null}
              <ToolTip title="Delete">
                <IconButton aria-label="Delete" onClick={handleDelete}>
                  <DeleteIcon/>
                </IconButton>
              </ToolTip>
            </div>
          ) : <div>
            <ToolTip title="过滤">
              <IconButton aria-label="Filter list">
                <FilterListIcon/>
              </IconButton>
            </ToolTip>
          </div>
        }
      </div>
    </Toolbar>
  )
}

TTOOLBAR.propTypes = {}

export default withStyles(styles)(TTOOLBAR)