import React, { useEffect, useState } from 'react'
import PropTypes from 'prop-types'
import { makeStyles } from '@material-ui/core/styles'
import classNames from 'classnames'

import THEAD from "./THEAD"
import TR from "./TR"
import TH from "./TH"
import TD from "./TD"
import TBODY from "./TBODY"
import TFOOT from "./TFOOT"

import TableRow from '@material-ui/core/TableRow'
import TableCell from '@material-ui/core/TableCell'

import EditIcon from "@material-ui/icons/Edit"
import DeleteIcon from "@material-ui/icons/Delete"
import PrintIcon from "@material-ui/icons/Print"
import OpenWithIcon from '@material-ui/icons/OpenWith'
import AddIcon from '@material-ui/icons/Add'
import RefreshIcon from '@material-ui/icons/Refresh'
import FilterListIcon from '@material-ui/icons/FilterList'
import ImportExportIcon from '@material-ui/icons/ImportExport'

import { lighten } from '@material-ui/core/styles/colorManipulator'

import Pagination from "../Pagination"

import Checkbox from "../_base/Checkbox"
import IconButton from "../_base/IconButton"
import Toolbar from "../_base/Toolbar"
import ToolTip from "../_base/Tooltip"
import TextField from "../_base/TextField"

import { sub, add } from './utils'

const useStyles = makeStyles(theme => ({
  root: {
    padding: theme.spacing(),
    backgroundColor: theme.palette.background.paper,
    border: `solid 1px ${theme.palette.text.hint}`,
  },
  table: {
    width: '100%',
    borderCollapse: 'collapse',
    border: `solid 1px ${theme.palette.text.hint}`,
  },
  highlight:
    theme.palette.type === 'light'
      ? {
        color: theme.palette.secondary.main,
        backgroundColor: lighten(theme.palette.secondary.light, 0.85),
      }
      : {
        color: theme.palette.text.primary,
        backgroundColor: theme.palette.secondary.dark,
      },
  spacer: {
    flex: '1 1 100%',
  },
  actions: {
    flex: '0 0 auto',
    color: theme.palette.text.secondary,
  },
  title: {
    flex: '0 0 auto',
  },
  noPadding: {
    padding: 0
  },
  // 工具栏样式
  toolbar: {
    borderStyle: 'dotted',
    borderColor: theme.palette.text.hint,
    borderWidth: 1,
    borderBottom: 'none',
  },
  thead: {
    borderTop: 'solid 1px',
    borderLeft: 'solid 1px',
    borderColor: theme.palette.text.hint,
  },
  tfoot: {
    borderBottom: `solid 1px ${theme.palette.text.hint}`,
    borderLeft: 'solid 1px',
    borderColor: theme.palette.text.hint,
  }
}))


/**
 * 表格高级组件
 *
 * 该组件使用基础组建来组织数据
 * 不涉及样式内容，样式由基础组件负责调整
 */
function Table(props) {
  const {
    // 不显显示序号
    noLineNumber,
    // 可以选择
    selectAble,
    // 没有数据时的提示语
    noDataTip,
    // 列信息
    columns,
    // 要呈现的数据
    data,
    // 已选中行数量
    numSelected,
    // 总行数
    rowCount,
    // 正处于移动状态
    inMoving,
    // 行单击方法
    handleRowClick,
    // 新增方法
    handleAdd,
    // 搜索方法
    handleSearch,
    // 处理选中事件
    handleSelect,
    handleSelectAll,
    // 编辑当前项方法
    handleEdit,
    // 删除当前行方法（通过行的操作按钮调用）
    handleDelete,
    // 处理移动的方法
    handleMove,
    // 删除选中项方法
    handleDeleteSelected,
    // 底部合计信息
    brief,
  } = props

  const classes = useStyles()

  // 载入提示
  const [loading, setLoaing] = useState(false)
  // 提示信息
  const [tip, setTip] = useState(null)
  // 当前页
  const [currentPage, setCurrentPage] = useState(1)
  // 远程请求参数
  const [params, setParams] = useState({})

  // 分页数据
  const [pagination, setPagination] = useState({})


  function componentWillReceiveProps(nextProps, nextContext) {
    //console.debug(props.request, nextProps.request)
    if (nextProps.data) {
      // 用于显示本地数据时，收到新数据需重置页码并更新现有显示数据
      setState({
        currentPage: 1,
        data: nextProps.data.slice(0, pageSize)
      })
    }
    else {
      if (JSON.stringify(props.request) !== JSON.stringify(nextProps.request)) {
        console.debug('Table received new props')
        setState({
          currentPage: 1,
          record_count: 0
        }, () => loadData())
      }
    }
  }

  useEffect(() => {
    if (props.request) {
      // 如果指定了查询参数，则进行远程数据调取
      if (url) {
        loadData()
      }
    }
  }, [])

  /**
   * 翻页事件
   * @param page
   */
  function handleChangePage(page) {
    // 本地数据模式下的翻页
    const start = (page - 1) * pageSize
    const end = start + pageSize
    setState({
      currentPage: page,
      data: data.slice(start, end)
    })
  }

  /**
   * 导出数据
   */
  function exportData() {
    // const { url, criteria } = props.request
    // request.get(url, { criteria, export: true }).then(res => {
    //   if (res.result === 0) {
    //     // 生成表头
    //     const realColumns = []
    //     const heads = []
    //     props.columns.map((column, i) => {
    //       // 如果是多层表头
    //       if (column.children) {
    //         // 记录合并列的详细信息，用于生成第二行
    //         column.children.map(child => {
    //           heads.push(child.title)
    //         })
    //         realColumns.push(child.name)
    //       }
    //       else {
    //         heads.push(column.title)
    //         realColumns.push(column.name)
    //       }
    //     })
    //     const headStr = heads.join(',') + '\n'
    //
    //     let str = ''
    //     res.data.map(data => {
    //       const row = []
    //       realColumns.map(realColumn => {
    //         row.push(data[realColumn])
    //       })
    //       str += row.join(',') + '\n'
    //     })
    //     let blob = new Blob(["\uFEFF" + headStr + str], {
    //       type: "text/plain;charset=utf-8"
    //     })
    //     FileSaver.saveAs(blob, "data.csv")
    //   }
    //   else {
    //     alert(res.message)
    //   }
    // }).catch(e => {
    //   alert(e.message)
    // })

  }

  console.debug('_com/Table render')


  // 真实要生成的列
  const _columns = []
  // 多层表头列
  const columnBands = []
  // 快整搜索列
  const searchColumns = []
  // 真实数据列
  const dataColumns = []
  // 真实列数
  let columnNum = 0

  // 如果允许选择，则列数+1
  if (selectAble) {
    columnNum += 1
  }

  // 遍历列，提取：多层表头/快速搜索列等信息
  columns.map((column, i) => {
    // 如果是多层表头
    if (column.children) {
      // 增加合并列，colSpan用于标识这是多层表头
      _columns.push({
        field: `ban${i}`,
        title: column.title,
        colSpan: column.children.length
      })

      // 记录合并列的详细信息，用于生成第二行
      column.children.map(child => {
        columnBands.push(child)
        dataColumns.push(child)
        if (child.searcher) {
          searchColumns.push(child)
        }
      })
    }
    else {
      _columns.push(column)
      dataColumns.push(column)
      if (column.searcher) {
        searchColumns.push(column)
      }

    }
  })
  columnNum += _columns.length

  // 增加序号列
  if (!noLineNumber) {
    const SnRow = {
      name: 'no',
      title: 'No',
      align: 'center'
    }

    _columns.unshift(SnRow)
    dataColumns.unshift(SnRow)

    columnNum += 1
  }

  // 增加操作列
  const showRowTools = handleEdit || handleDelete
  if (showRowTools) {
    columnNum += 1
  }

  // 根据是否有多层表头，重新处理一遍列数据
  const banded = columnBands.length > 0
  if (banded) {
    _columns.map(column => {
      if (!column.colSpan) {
        column.rowSpan = 2
      }
    })
  }
  const head =
    <THEAD>
    <TableRow className={classes.thead}>
      {
        !selectAble ? null : <TH style={{ padding: 0 }} rowSpan={banded ? 2 : null}>
          <Checkbox
            className={classes.noPadding}
            id="TableCheckAll"
            indeterminate={numSelected > 0 && numSelected < rowCount}
            checked={numSelected === rowCount}
            onChange={handleSelectAll}
          />
        </TH>
      }
      {
        _columns.map(column => {
          return (
            <TH
              key={column.title}
              rowSpan={column.rowSpan || null}
              colSpan={column.colSpan || null}
            >
              {column.title}
            </TH>
          )
        })
      }
      {
        showRowTools ? <TH align="center">操作</TH> : null
      }
    </TableRow>
    {
      columnBands.length > 0 ?
        <TableRow className={classes.thead}>
          {
            columnBands.map(column => {
              return <TH key={column.field}>{column.title}</TH>
            })
          }
        </TableRow> : null
    }
    </THEAD>

  // 合计
  const briefs = {}
  const tbody = data.length > 0 ?
    <TBODY>
    {
      data.map((row, i) => {
        // 有row为undefined/null的可能
        if (!row) return

        if (!noLineNumber) {
          row.no = i + 1
        }
        return (
          <TR
            hover="true"
            key={row.guid || i}
            style={{ cursor: handleRowClick && 'pointer' }}
            onClick={handleRowClick ? event => handleRowClick(row) : null}
          >
            {
              // 生成选择列
              selectAble && <TD
                align="center">
                <Checkbox
                  style={{ padding: 0 }}
                  onChange={event => handleSelect(event, row)}
                />
              </TD>
            }
            {
              dataColumns.map(column => {
                // 生成正式数据列
                const {
                  // 列键值string
                  field,
                  // 对齐方式:left/center/right
                  align,
                  // 后缀string
                  suffix,
                  // 根据提供的对象显示值：{'${company_guid}': '一店'}
                  lookup
                } = column
                // 提供左右对齐则按提供值，不提供则数字向右，文本向左对齐

                // 生成页脚汇总数据
                if (column.brief) {
                  const { type } = column.brief
                  switch (type) {
                    // 合计值汇总
                    case 'sum':
                      if (!isNaN(row[column.field])) {
                        briefs[column.field] = add(briefs[column.field] || 0, row[column.field])
                      }
                      break

                    // TODO 平均值汇总
                    case 'avarage':

                      break
                  }
                }

                // 提定了替代内容时，显示替代内容
                const text = lookup ? lookup[row[field]] : row[field]
                column.align = align || (isNaN(text) ? 'left' : 'right')
                // 生成数据单元格
                return (
                  <TD
                    key={`${row.guid}${field}`}
                    align={column.align}
                  >
                    {text}{suffix}
                  </TD>
                )
              })
            }
            {
              showRowTools && <TD key={`${row.guid}tools`} align="center">
                {
                  // 生成操作栏
                  handleEdit ?
                    <IconButton className={classes.noPadding} onClick={() => handleEdit(row)}><EditIcon
                      fontSize="small"/></IconButton> : null
                }
                {
                  handleDelete ?
                    <IconButton className={classes.noPadding} onClick={() => handleDelete(row)}><DeleteIcon
                      fontSize="small"/></IconButton> : null
                }
              </TD>
            }
          </TR>
        )
      })
    }
    </TBODY> :
    <TBODY>
    <TableRow>
      <TableCell colSpan={dataColumns.length + (selectAble ? 1 : 0)}>{noDataTip || tip}</TableCell>
    </TableRow>
    </TBODY>

  // 需传递给工具条的属性
  const showToolbar = handleAdd || handleDelete
  const toolbar = !showToolbar ? null : <div className="noprint"><Toolbar
    className={classNames(classes.toolbar, {
      [classes.highlight]: numSelected > 0,
    })}
  >
    <div className={classes.title}>
      {
        searchColumns.map((column, i) => {
          return (
            <TextField
              key={'searcher' + i}
              style={{ marginRight: 8 }}
              label={column.title}
              autoFocus={i === 0}
              onKeyDown={e => {
                if (e.keyCode === 13) {
                  column.searcher.handler(e.target.value)
                }
              }}
            />
          )
        })
      }
    </div>
    <div className={classes.spacer}/>
    <div className={classes.actions}>
      {
        numSelected > 0 ? (
          <div>
            <ToolTip title="移动">
              <IconButton aria-label="Move" onClick={handleMove}>
                <OpenWithIcon/>
              </IconButton>
            </ToolTip>
            {inMoving ? <span style={{ animation: 'cursorjump 1s infinite forwards' }}>请选择目标分类……</span> : null}
            <ToolTip title="Delete">
              <IconButton aria-label="Delete" onClick={handleDelete}>
                <DeleteIcon/>
              </IconButton>
            </ToolTip>
          </div>
        ) : <div>
          {
            handleAdd ?
              <ToolTip title="新增">
                <IconButton aria-label="Move" onClick={handleAdd}>
                  <AddIcon/>
                </IconButton>
              </ToolTip> : null
          }
          {
            request ? <ToolTip title="刷新">
              <IconButton onClick={loadData}>
                <RefreshIcon/>
              </IconButton>
            </ToolTip> : null
          }
          <ToolTip title="过滤">
            <IconButton aria-label="Filter list">
              <FilterListIcon/>
            </IconButton>
          </ToolTip>
          {
            <ToolTip title="导出">
              <IconButton aria-label="export" onClick={exportData}>
                <ImportExportIcon/>
              </IconButton>
            </ToolTip>
          }
          <ToolTip title="打印">
            <IconButton aria-label="Print" onClick={window.print}>
              <PrintIcon/>
            </IconButton>
          </ToolTip>
        </div>
      }
    </div>
  </Toolbar>
  </div>

  let foot
  if (Object.getOwnPropertyNames(briefs).length > 0) {
    // 生成数据汇总行
    const domFoot = []
    if (selectAble) {
      domFoot.push(<TD></TD>)
    }
    dataColumns.map((column, i) => {
      domFoot.push(
        <TD key={`foot${i}`} align={column.align || null}>{column.brief ? briefs[column.field] : null}</TD>
      )
    })

    // 表尾
    foot = <TFOOT>
    <TableRow className={classes.tfoot}>
      {domFoot}
    </TableRow>
    </TFOOT>
  }
  else {
    foot = null
  }

  // 分页栏，只有传入有效的分页属性才显示分页栏
  // 生成分页数据
  let recordLength = data ? data.length : 0 // 总数据行数

  // 生成分页组件
  const domPagination = Object.getOwnPropertyNames(pagination).length > 0 ? <Pagination
    current={currentPage}
    pageSize={pageSize}
    recordCount={recordLength}
    handleChangePage={handleChangePage}
  /> : null

  return (
    <div className={classes.root}>
      {toolbar}
      <table className={classes.table}>
        {head}
        {tbody}
        {foot}
      </table>
      {domPagination}
    </div>
  )
}

Table.propTypes = {
  noLineNumber: PropTypes.bool,
  selectAble: PropTypes.bool,
  columns: PropTypes.array.isRequired,
  data: PropTypes.array,
  handleRowClick: PropTypes.func,
  handleDelete: PropTypes.func,
  handleAdd: PropTypes.func
}

export default Table
